"""High level OpenERP assertions and helpers."""


class OpenErpAssertions(object):
    """Mixin class providing assertion and helper methods to write tests.

    Some of these methods will make less sense with the new API scheduled for
    OpenERP 8, we'll provide them as simple wrappers for backwards compat then.
    """

    def _model_inst(self, model):
        return self.registry(model) if isinstance(model, basestring) else model

    def searchUnique(self, model, domain, **kw):
        """Search, assert that there's exactly one result, return its id.

        :model: can be either a string (model name) or a model instance
        :domain: domain to perform the search with
        :kw: any additional keyword (e.g., ``context``) is passed both
             to search or read.
        """
        model = self._model_inst(model)
        ids = model.search(self.cr, self.uid, domain, **kw)
        self.assertEquals(len(ids), 1,
                          msg="Domain %r on model %r should give "
                          "exactly one record "
                          "(got %d)" % (domain, model, len(ids)))
        return ids[0]

    def readUnique(self, model, domain, load='_classic_write', **kw):
        """Fetch a record by domain, assert unicity, return a read.

        :model: can be either a string (model name) or a model instance
        :load: directly passed to the underlying ``read`` call.
        :kw: any additional keyword (e.g., ``context``) is passed both
             to search or read.
        """
        model = self._model_inst(model)
        rec_id = self.searchUnique(model, domain, **kw)

        return model.read(self.cr, self.uid, rec_id, load=load, **kw)

    def assertRecord(self, model, rec_id, vals, list_to_set=False, **kw):
        """Fetch and compare a record with the provided values dict.

        This saves a LOT of typing and makes assertion much easier to read
        while avoiding a browse.

        :model: can be either a string (model name) or a model instance
        :rec_id: numeric id of the record to check
        :vals: ``dict`` of expected values. For ``many2one`` fields, just pass
               the expected numeric id.
        :list_to_set: if ``True``, all ``list`` instances in the read record
                      will be converted to ``set`` prior tocomparison
        :kw: any additional keyword arguments are passed on to the underlying
             call(s) of ``assertEqual``.

        Examples:

         - if you already have the model obj as ``users_model``::

             self.assertRecord(users_model, 1, dict(name='Administrator'))

         - in general::

             self.assertRecord('res.users', 1, dict(name='Administrator',
                                                    partner_id=1))
        """
        def normalize(v):
            if list_to_set and isinstance(v, list):
                return set(v)
            return v

        read = dict((k, normalize(v))
                    for k, v in self._model_inst(model).read(
                        self.cr, self.uid, rec_id, vals.keys(),
                        load='_classic_write').iteritems()
                    if k in vals)
        self.assertEqual(read, vals, **kw)

    def assertNoRecord(self, model, domain, context=None, msg=None):
        """Assert that the given domain gives 0 results on prescribed model.

        :param model: a :class:``osv.Model`` instance or a model name
        :param domain: a standard OpenERP domain
        :param context: optional OpenERP context
        :param msg: same meaning as for :meth:assertEqual
        """
        return self.assertEqual(
            len(self._model_inst(model).search(self.cr, self.uid,
                                               domain, context=context)),
            0, msg=msg)



from anybox.testing.openerp import SharedSetupTransactionCase


class TestSharedSetup(SharedSetupTransactionCase):
    """Proof that the SharedSetup class works."""

    _data_files = ('sharedsetup.xml',
                   )

    _module_ns = 'sharedtest'

    def setUp(self):
        super(TestSharedSetup, self).setUp()
        self.model = self.registry('res.partner')

    def assertRollbacked(self):
        """Proof that data load and rollback works."""
        rec = self.browse_ref('sharedtest.partner')
        self.assertEquals(rec.name, 'Tests')
        return rec.id

    def test_one(self):
        rec_id = self.assertRollbacked()
        self.model.write(self.cr, self.uid, rec_id, dict(name="spam"))
        # write has really been done
        self.assertRecord(self.model, rec_id, dict(name="spam"))

    def test_two(self):
        rec_id = self.assertRollbacked()
        self.model.write(self.cr, self.uid, rec_id, dict(name="eggs"))
        # write has really been done
        self.assertRecord(self.model, rec_id, dict(name="eggs"))



from anybox.testing.openerp import TransactionCase

import random


class TestAssertions(TransactionCase):
    """Proof that the high level assertions do work."""

    def setUp(self):
        super(TestAssertions, self).setUp()
        # we could do this in a SharedSetupTransactionCase, but the point here
        # is to test the helper methods in the simplest context
        model = self.model = self.registry('res.partner')
        self.rec_id = model.create(self.cr, self.uid, dict(name="Assertor"))

    def test_assertRecord_model_inst(self):
        self.assertRecord(self.model, self.rec_id, dict(name="Assertor"))
        self.assertRaises(AssertionError, self.assertRecord,
                          self.model, self.rec_id, dict(name="Assertors"))

    def test_assertRecord_model_name(self):
        self.assertRecord('res.partner', self.rec_id, dict(name="Assertor"))

    def test_assertRecord_list_to_set(self):
        model, cr, uid, rec_id = self.model, self.cr, self.uid, self.rec_id

        # doing with bank_ids, any other o2m or m2m would do. If this
        # one gets too complicated (should find another one already, but
        # good enough for now)
        model.write(cr, uid, rec_id,
                    dict(bank_ids=[(0, 0, dict(state='', acc_number=1)),
                                   (0, 0, dict(state='', acc_number=2)),
                                   (0, 0, dict(state='', acc_number=3))]))
        bk_ids = self.registry('res.partner.bank').search(
            cr, uid, [('partner_id', '=', rec_id)])
        self.assertTrue(bk_ids)

        random.shuffle(bk_ids)  # for good measure
        self.assertRecord(model, rec_id, dict(bank_ids=set(bk_ids)),
                          list_to_set=True)
        self.assertRaises(AssertionError, self.assertRecord,
                          model, rec_id, dict(bank_id=set(bk_ids[1:])),
                          list_to_set=True)

    def test_searchUnique(self):
        for model in (self.model, 'res.partner'):
            self.assertEqual(
                self.searchUnique(model, [('name', '=', 'Assertor')]),
                self.rec_id)

    def test_readUnique(self):
        for model in (self.model, 'res.partner'):
            r = self.readUnique(model, [('name', '=', 'Assertor')])
            self.assertEqual(r['id'], self.rec_id)
            self.assertEqual(r['name'], 'Assertor')



"""This module provides rollbacked tests with shared data loaded once.
"""
import os
import sys
from .assertions import OpenErpAssertions
from openerp.tests.common import SingleTransactionCase
from openerp.tools.convert import convert_xml_import


class SharedSetupTransactionCase(OpenErpAssertions, SingleTransactionCase):
    """Insulated tests with a shared data setup.

    Loads data and plays the tests within a single transaction.
    XML files listed in _data_files class attribute are loaded.
    The path is interpreted relative to the package in which the subclass sits.

    For example, for a class defined in MODULE/tests/test_spam,
    _data_files = ('data/spam.xml',)
    will load MODULE/tests/data/spam.xml

    The path separator is always / for convenience on POSIX systems.
    On operating systems with different path separators,
    an automatic conversion is done.

    A rollback to savepoint is issued after each test.
    The whole transaction is rollbacked at class teardown
    """

    _savepoint_name = None

    _data_files = ()

    _module_ns = 'tests'  # indicate in which module's namespace the loaded data should be put

    @classmethod
    def savepoint_name(cls):
        name = cls._savepoint_name
        return name if name is not None else cls.__name__

    @classmethod
    def setUpClass(cls):
        SingleTransactionCase.setUpClass()
        # Any exception from here would cause tear downs not to
        # be executed.
        # This has been witnessed to cause PG to deadlock,
        # a major developper headache and lost time
        try:
            cls.initTestData()
        except:
            cls.tearDownClass()
            raise

        cls.cr.execute('SAVEPOINT "%s"' % cls.savepoint_name())

    @classmethod
    def initTestData(cls):
        """Loads the data file before the savepoint.

        Subclasses can override this, and possibly call super() to still benefit from the data
        loading facility."""
        module = sys.modules[cls.__module__]
        base_path = os.path.dirname(module.__file__)
        for path in cls._data_files:
            path = path.split('/')
            path.insert(0, base_path)
            path = os.path.join(*path)
            convert_xml_import(cls.cr, cls._module_ns, path)  # TODO YML, CSV ?

    def tearDown(self):
        # GR: direct API for ROLLBACK TO SAVEPOINT on cnx object ?
        self.cr.execute('ROLLBACK TO SAVEPOINT "%s"' % self.savepoint_name())

    @classmethod
    def tearDownClass(cls):
        cls.registry('ir.model.data').clear_caches()
        super(SharedSetupTransactionCase, cls).tearDownClass()



import os
import sys
from setuptools import setup, find_packages

version = '1.2.1'

if sys.version_info < (2, 6):
    sys.stderr.write("This package requires Python 2.6 or newer. "
                     "Yours is " + sys.version + os.linesep)
    sys.exit(1)

requires = ['setuptools',
            ]

setup(
    name="anybox.testing.openerp",
    version=version,
    author="Anybox",
    author_email="contact@anybox.fr",
    description="Useful testing base classes and tools for OpenERP",
    license="AGPL v3+",
    long_description='\n'.join((
        open('README.rst').read(),
        open('CHANGES.rst').read())),
    url="https://bitbucket.org/anybox/anybox.testing.openerp",
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=requires,
    tests_require=requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    entry_points={},
)



import sys
from .assertions import OpenErpAssertions
from openerp.tests.common import TransactionCase as OpenErpTransactionCase


class TransactionCase(OpenErpAssertions, OpenErpTransactionCase):

    _transaction_case_teared_down = False

    def setUp(self):
        if sys.version_info >= (2, 7):
            # handling same risks as in sharedsetup
            self.addCleanup(self.forceTearDown)
        super(TransactionCase, self).setUp()

    def forceTearDown(self):
        if not self._transaction_case_teared_down:
            self.tearDown()

    def tearDown(self):
        super(TransactionCase, self).tearDown()
        self._transaction_case_teared_down = True
